@@ -9,8 +9,6 @@
  * except according to those terms.
  */
 
-use std::borrow::Cow;
-
 use super::{
     multi_byte::*,
     single_byte::*,
@@ -29,28 +27,21 @@ pub fn get_charset_decoder<'x>(charset: &[u8]) -> Option<DecoderFnc<'x>> {
             } else {
                 *ch
             };
-            // SAFE: `pos` is always lower than 45
-            unsafe {
-                *l_charset.get_unchecked_mut(pos) = ch;
-            }
+            l_charset[pos] = ch;
             if let 0 | 3 | 6 | 7 | 8 | 9 = pos {
-                // SAFE: CH_HASH's size is u8::MAX
-                hash += unsafe { *CH_HASH.get_unchecked(ch as usize) };
+                hash += CH_HASH[ch as usize];
             }
             if pos == charset.len() - 1 {
-                // SAFE: CH_HASH's size is u8::MAX
-                hash += unsafe { *CH_HASH.get_unchecked(ch as usize) };
+                hash += CH_HASH[ch as usize];
             }
         }
 
         if (7..=764).contains(&hash) {
             let hash = (hash - 7) as usize;
-            // SAFE: CH_MAP's size is 758
-            let &ch_map = unsafe { CH_MAP.get_unchecked(hash) };
+            let ch_map = CH_MAP[hash];
 
             if l_charset[..charset.len()].eq(ch_map) {
-                // SAFE: FNC_MAP's size is 758
-                return Some(unsafe { *FNC_MAP.get_unchecked(hash) });
+                return Some(FNC_MAP[hash]);
             }
         }
     }
@@ -58,8 +49,8 @@ pub fn get_charset_decoder<'x>(charset: &[u8]) -> Option<DecoderFnc<'x>> {
     None
 }
 
-pub fn decoder_default(bytes: &[u8]) -> Cow<str> {
-    String::from_utf8_lossy(bytes)
+pub fn no_op(_bytes: &[u8]) -> String {
+    "".to_string()
 }
 
 #[cfg(test)]
@@ -70,9 +61,9 @@ mod tests {
     fn get_decoder_charset() {
         let inputs = [
             "l8",
-            "utf-8",
+            //"utf-8",
             "utf-7",
-            "US-Ascii",
+            //"US-Ascii",
             "csgb18030",
             "iso-8859-1",
             "extended_unix_code_packed_format_for_japanese",
@@ -115,15 +106,15 @@ static CH_MAP: &[&[u8]; 758] = &[
     b"l1",
     b"",
     b"",
-    b"utf-8",
+    b"", //b"utf-8",
     b"latin1",
-    b"us",
+    b"", //b"us",
     b"",
     b"",
     b"",
     b"koi8-r",
     b"l5",
-    b"us-ascii",
+    b"", //b"us-ascii",
     b"",
     b"",
     b"latin5",
@@ -174,7 +165,7 @@ static CH_MAP: &[&[u8]; 758] = &[
     b"euc-jp",
     b"latin-9",
     b"",
-    b"iso646-us",
+    b"", //b"iso646-us",
     b"iso_8859-8:1988",
     b"iso-8859-15",
     b"",
@@ -198,7 +189,7 @@ static CH_MAP: &[&[u8]; 758] = &[
     b"",
     b"iso-8859-14",
     b"l2",
-    b"iso-ir-6",
+    b"", //b"iso-ir-6",
     b"",
     b"iso_8859-5",
     b"latin2",
@@ -236,7 +227,7 @@ static CH_MAP: &[&[u8]; 758] = &[
     b"l10",
     b"",
     b"iso_8859-9:1989",
-    b"iso_646.irv:1991",
+    b"", //b"iso_646.irv:1991",
     b"windows-1251",
     b"asmo-708",
     b"",
@@ -337,11 +328,11 @@ static CH_MAP: &[&[u8]; 758] = &[
     b"",
     b"iso-8859-2",
     b"arabic",
-    b"csascii",
+    b"", //b"csascii",
     b"",
     b"",
     b"",
-    b"csutf8",
+    b"", //b"csutf8",
     b"cseuckr",
     b"",
     b"macintosh",
@@ -364,7 +355,7 @@ static CH_MAP: &[&[u8]; 758] = &[
     b"",
     b"latin10",
     b"",
-    b"ansi_x3.4-1968",
+    b"", //b"ansi_x3.4-1968",
     b"cp819",
     b"windows-936",
     b"tis-620",
@@ -401,7 +392,7 @@ static CH_MAP: &[&[u8]; 758] = &[
     b"cswindows1256",
     b"",
     b"",
-    b"ibm367",
+    b"", //b"ibm367",
     b"",
     b"",
     b"",
@@ -409,7 +400,7 @@ static CH_MAP: &[&[u8]; 758] = &[
     b"csiso885915",
     b"",
     b"",
-    b"ansi_x3.4-1986",
+    b"", //b"ansi_x3.4-1986",
     b"iso-ir-157",
     b"csisolatin4",
     b"",
@@ -515,7 +506,7 @@ static CH_MAP: &[&[u8]; 758] = &[
     b"",
     b"",
     b"",
-    b"cp367",
+    b"", //b"cp367",
     b"",
     b"",
     b"",
@@ -868,763 +859,763 @@ static CH_MAP: &[&[u8]; 758] = &[
 ];
 
 #[allow(clippy::type_complexity)]
-static FNC_MAP: &[for<'x> fn(&'x [u8]) -> Cow<'x, str>; 758] = &[
+static FNC_MAP: &[for<'x> fn(&'x [u8]) -> String; 758] = &[
     decoder_iso_8859_14,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_14,
     decoder_iso_8859_1,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
     decoder_utf8,
     decoder_iso_8859_1,
     decoder_utf8,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
     decoder_koi8_r,
     decoder_iso_8859_9,
     decoder_utf8,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
     decoder_iso_8859_9,
-    decoder_default,
+    no_op,
     decoder_shift_jis,
     decoder_shift_jis,
-    decoder_default,
+    no_op,
     decoder_koi8_u,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
     decoder_big5,
-    decoder_default,
+    no_op,
     decoder_iso_8859_1,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
     decoder_euc_kr,
     decoder_iso_8859_4,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_4,
-    decoder_default,
+    no_op,
     decoder_ibm866,
-    decoder_default,
+    no_op,
     decoder_iso_8859_9,
     decoder_ibm866,
     decoder_iso_8859_10,
     decoder_iso_8859_7,
-    decoder_default,
+    no_op,
     decoder_iso_8859_8,
     decoder_iso_8859_10,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
     decoder_utf16,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_1,
     decoder_tis_620,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_8,
     decoder_euc_jp,
     decoder_iso_8859_15,
-    decoder_default,
+    no_op,
     decoder_utf8,
     decoder_iso_8859_8,
     decoder_iso_8859_15,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_1,
     decoder_iso_8859_14,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_5,
     decoder_iso_8859_16,
-    decoder_default,
+    no_op,
     decoder_utf16_be,
-    decoder_default,
+    no_op,
     decoder_iso_8859_5,
     decoder_iso_8859_15,
-    decoder_default,
+    no_op,
     decoder_utf16_le,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
     decoder_iso_8859_14,
     decoder_iso_8859_2,
     decoder_utf8,
-    decoder_default,
+    no_op,
     decoder_iso_8859_5,
     decoder_iso_8859_2,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_14,
     decoder_iso_8859_16,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_4,
     decoder_iso_8859_14,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_9,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_5,
     decoder_iso_8859_16,
-    decoder_default,
+    no_op,
     decoder_iso_8859_6,
-    decoder_default,
+    no_op,
     decoder_iso_8859_4,
     decoder_iso_8859_8,
-    decoder_default,
+    no_op,
     decoder_ibm_850,
-    decoder_default,
+    no_op,
     decoder_iso_8859_9,
     decoder_ibm_850,
     decoder_cp1258,
     decoder_iso_8859_16,
-    decoder_default,
+    no_op,
     decoder_iso_8859_9,
     decoder_utf8,
     decoder_cp1251,
     decoder_iso_8859_6,
-    decoder_default,
+    no_op,
     decoder_iso_8859_4,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
     decoder_iso_8859_7,
-    decoder_default,
+    no_op,
     decoder_iso_8859_6,
-    decoder_default,
+    no_op,
     decoder_cp1255,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
     decoder_iso_8859_2,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
     decoder_gbk,
-    decoder_default,
+    no_op,
     decoder_utf7,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_6,
-    decoder_default,
+    no_op,
     decoder_iso_8859_3,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_3,
     decoder_cp1254,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
     decoder_iso_8859_8,
     decoder_iso_8859_10,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_7,
     decoder_cp1256,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_10,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_7,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_7,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_3,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
     decoder_gbk,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
     decoder_windows874,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_13,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_1,
-    decoder_default,
+    no_op,
     decoder_cp1252,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
     decoder_euc_jp,
     decoder_iso2022_jp,
-    decoder_default,
+    no_op,
     decoder_macintosh,
-    decoder_default,
+    no_op,
     decoder_iso_8859_3,
-    decoder_default,
+    no_op,
     decoder_koi8_r,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
     decoder_iso_8859_2,
     decoder_iso_8859_6,
     decoder_utf8,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
     decoder_utf8,
     decoder_euc_kr,
-    decoder_default,
+    no_op,
     decoder_macintosh,
     decoder_gbk,
     decoder_big5,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_2,
-    decoder_default,
+    no_op,
     decoder_koi8_u,
     decoder_cp1258,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
     decoder_cp1250,
     decoder_cp1251,
-    decoder_default,
+    no_op,
     decoder_iso_8859_6,
-    decoder_default,
+    no_op,
     decoder_iso_8859_16,
-    decoder_default,
+    no_op,
     decoder_utf8,
     decoder_iso_8859_1,
     decoder_gbk,
     decoder_tis_620,
     decoder_cp1255,
-    decoder_default,
+    no_op,
     decoder_iso_8859_4,
-    decoder_default,
+    no_op,
     decoder_cp1257,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_16,
     decoder_iso_8859_1,
     decoder_windows874,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_8,
     decoder_cp1253,
     decoder_cp1254,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
     decoder_iso_8859_9,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_7,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
     decoder_cp1256,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
     decoder_utf8,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_2,
     decoder_iso_8859_15,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
     decoder_utf8,
     decoder_iso_8859_10,
     decoder_iso_8859_4,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
     decoder_euc_jp,
     decoder_ibm866,
     decoder_iso_8859_10,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_14,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
     decoder_utf16,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
     decoder_iso_8859_7,
     decoder_iso_8859_16,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
     decoder_cp1252,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_7,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_7,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
     decoder_utf16_be,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
     decoder_utf16_le,
     decoder_iso_8859_3,
     decoder_iso_8859_2,
-    decoder_default,
+    no_op,
     decoder_ibm866,
-    decoder_default,
+    no_op,
     decoder_ibm_850,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
     decoder_shift_jis,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
     decoder_cp1250,
-    decoder_default,
+    no_op,
     decoder_iso_8859_3,
     decoder_utf7,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_6,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_1,
     decoder_macintosh,
     decoder_gb18030,
     decoder_cp1257,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
     decoder_cp1253,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
     decoder_utf8,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
     decoder_gbk,
     decoder_iso_8859_3,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_13,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_14,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_6,
-    decoder_default,
+    no_op,
     decoder_iso_8859_5,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
     decoder_tis_620,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso2022_jp,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
     decoder_ibm_850,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
     decoder_iso_8859_5,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
-    decoder_default,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
+    no_op,
     decoder_gb18030,
 ];
