@@ -34,8 +34,8 @@
 //! - **Fast parsing** of message header fields, character set names and HTML entities using [perfect hashing](https://en.wikipedia.org/wiki/Perfect_hash_function).
 //! - Written in **100% safe** Rust with no external dependencies.
 //! - Every function in the library has been [fuzzed](#testing-fuzzing--benchmarking) and
-//!   meticulously [tested with MIRI](#testing-fuzzing--benchmarking).
-//! - Thoroughly **battle-tested** with millions of real-world e-mail messages dating from 1995 until today.
+//!   thoroughly [tested with MIRI](#testing-fuzzing--benchmarking).
+//! - **Battle-tested** with millions of real-world e-mail messages dating from 1995 until today.
 //!
 //! Jump to the [example](#usage-example).
 //!
@@ -112,42 +112,46 @@
 //! ```
 //!    use mail_parser::*;
 //!
-//!    let input = concat!(
-//!        "From: Art Vandelay <art@vandelay.com> (Vandelay Industries)\n",
-//!        "To: \"Colleagues\": \"James Smythe\" <james@vandelay.com>; Friends:\n",
-//!        "    jane@example.com, =?UTF-8?Q?John_Sm=C3=AEth?= <john@example.com>;\n",
-//!        "Date: Sat, 20 Nov 2021 14:22:01 -0800\n",
-//!        "Subject: Why not both importing AND exporting? =?utf-8?b?4pi6?=\n",
-//!        "Content-Type: multipart/mixed; boundary=\"festivus\";\n\n",
-//!        "--festivus\n",
-//!        "Content-Type: text/html; charset=\"us-ascii\"\n",
-//!        "Content-Transfer-Encoding: base64\n\n",
-//!        "PGh0bWw+PHA+SSB3YXMgdGhpbmtpbmcgYWJvdXQgcXVpdHRpbmcgdGhlICZsZHF1bztle\n",
-//!        "HBvcnRpbmcmcmRxdW87IHRvIGZvY3VzIGp1c3Qgb24gdGhlICZsZHF1bztpbXBvcnRpbm\n",
-//!        "cmcmRxdW87LDwvcD48cD5idXQgdGhlbiBJIHRob3VnaHQsIHdoeSBub3QgZG8gYm90aD8\n",
-//!        "gJiN4MjYzQTs8L3A+PC9odG1sPg==\n",
-//!        "--festivus\n",
-//!        "Content-Type: message/rfc822\n\n",
-//!        "From: \"Cosmo Kramer\" <kramer@kramerica.com>\n",
-//!        "Subject: Exporting my book about coffee tables\n",
-//!        "Content-Type: multipart/mixed; boundary=\"giddyup\";\n\n",
-//!        "--giddyup\n",
-//!        "Content-Type: text/plain; charset=\"utf-16\"\n",
-//!        "Content-Transfer-Encoding: quoted-printable\n\n",
-//!        "=FF=FE=0C!5=D8\"=DD5=D8)=DD5=D8-=DD =005=D8*=DD5=D8\"=DD =005=D8\"=\n",
-//!        "=DD5=D85=DD5=D8-=DD5=D8,=DD5=D8/=DD5=D81=DD =005=D8*=DD5=D86=DD =\n",
-//!        "=005=D8=1F=DD5=D8,=DD5=D8,=DD5=D8(=DD =005=D8-=DD5=D8)=DD5=D8\"=\n",
-//!        "=DD5=D8=1E=DD5=D80=DD5=D8\"=DD!=00\n",
-//!        "--giddyup\n",
-//!        "Content-Type: image/gif; name*1=\"about \"; name*0=\"Book \";\n",
-//!        "              name*2*=utf-8''%e2%98%95 tables.gif\n",
-//!        "Content-Transfer-Encoding: Base64\n",
-//!        "Content-Disposition: attachment\n\n",
-//!        "R0lGODlhAQABAIAAAAAAAP///yH5BAEAAAAALAAAAAABAAEAAAIBRAA7\n",
-//!        "--giddyup--\n",
-//!        "--festivus--\n",
-//!    )
-//!    .as_bytes();
+//!    let input = br#"From: Art Vandelay <art@vandelay.com> (Vandelay Industries)
+//!To: "Colleagues": "James Smythe" <james@vandelay.com>; Friends:
+//!    jane@example.com, =?UTF-8?Q?John_Sm=C3=AEth?= <john@example.com>;
+//!Date: Sat, 20 Nov 2021 14:22:01 -0800
+//!Subject: Why not both importing AND exporting? =?utf-8?b?4pi6?=
+//!Content-Type: multipart/mixed; boundary="festivus";
+//!
+//!--festivus
+//!Content-Type: text/html; charset="us-ascii"
+//!Content-Transfer-Encoding: base64
+//!
+//!PGh0bWw+PHA+SSB3YXMgdGhpbmtpbmcgYWJvdXQgcXVpdHRpbmcgdGhlICZsZHF1bztle
+//!HBvcnRpbmcmcmRxdW87IHRvIGZvY3VzIGp1c3Qgb24gdGhlICZsZHF1bztpbXBvcnRpbm
+//!cmcmRxdW87LDwvcD48cD5idXQgdGhlbiBJIHRob3VnaHQsIHdoeSBub3QgZG8gYm90aD8
+//!gJiN4MjYzQTs8L3A+PC9odG1sPg==
+//!--festivus
+//!Content-Type: message/rfc822
+//!
+//!From: "Cosmo Kramer" <kramer@kramerica.com>
+//!Subject: Exporting my book about coffee tables
+//!Content-Type: multipart/mixed; boundary="giddyup";
+//!
+//!--giddyup
+//!Content-Type: text/plain; charset="utf-16"
+//!Content-Transfer-Encoding: quoted-printable
+//!
+//!=FF=FE=0C!5=D8"=DD5=D8)=DD5=D8-=DD =005=D8*=DD5=D8"=DD =005=D8"=
+//!=DD5=D85=DD5=D8-=DD5=D8,=DD5=D8/=DD5=D81=DD =005=D8*=DD5=D86=DD =
+//!=005=D8=1F=DD5=D8,=DD5=D8,=DD5=D8(=DD =005=D8-=DD5=D8)=DD5=D8"=
+//!=DD5=D8=1E=DD5=D80=DD5=D8"=DD!=00
+//!--giddyup
+//!Content-Type: image/gif; name*1="about "; name*0="Book ";
+//!              name*2*=utf-8''%e2%98%95 tables.gif
+//!Content-Transfer-Encoding: Base64
+//!Content-Disposition: attachment
+//!
+//!R0lGODlhAQABAIAAAAAAAP///yH5BAEAAAAALAAAAAABAAEAAAIBRAA7
+//!--giddyup--
+//!--festivus--
+//!"#;
 //!
 //!    let message = Message::parse(input).unwrap();
 //!
@@ -198,7 +202,7 @@
 //!
 //!    // HTML and text body parts are returned conforming to RFC8621, Section 4.1.4
 //!    assert_eq!(
-//!        message.get_html_body(0).unwrap().to_string(),
+//!        message.get_html_body(0).unwrap(),
 //!        concat!(
 //!            "<html><p>I was thinking about quitting the &ldquo;exporting&rdquo; to ",
 //!            "focus just on the &ldquo;importing&rdquo;,</p><p>but then I thought,",
@@ -208,7 +212,7 @@
 //!
 //!    // HTML parts are converted to plain text (and viceversa) when missing
 //!    assert_eq!(
-//!        message.get_text_body(0).unwrap().to_string(),
+//!        message.get_text_body(0).unwrap(),
 //!        concat!(
 //!            "I was thinking about quitting the “exporting” to focus just on the",
 //!            " “importing”,\nbut then I thought, why not do both? ☺\n"
@@ -216,10 +220,11 @@
 //!    );
 //!
 //!    // Supports nested messages as well as multipart/digest
-//!    let nested_message = match message.get_attachment(0).unwrap() {
-//!        MessagePart::Message(v) => v,
-//!        _ => unreachable!(),
-//!    };
+//!    let nested_message = message
+//!        .get_attachment(0)
+//!        .unwrap()
+//!        .unwrap_message()
+//!        .get_body();
 //!
 //!    assert_eq!(
 //!        nested_message.get_subject().unwrap(),
@@ -228,18 +233,15 @@
 //!
 //!    // Handles UTF-* as well as many legacy encodings
 //!    assert_eq!(
-//!        nested_message.get_text_body(0).unwrap().to_string(),
+//!        nested_message.get_text_body(0).unwrap(),
 //!        "ℌ𝔢𝔩𝔭 𝔪𝔢 𝔢𝔵𝔭𝔬𝔯𝔱 𝔪𝔶 𝔟𝔬𝔬𝔨 𝔭𝔩𝔢𝔞𝔰𝔢!"
 //!    );
 //!    assert_eq!(
-//!        nested_message.get_html_body(0).unwrap().to_string(),
+//!        nested_message.get_html_body(0).unwrap(),
 //!        "<html><body>ℌ𝔢𝔩𝔭 𝔪𝔢 𝔢𝔵𝔭𝔬𝔯𝔱 𝔪𝔶 𝔟𝔬𝔬𝔨 𝔭𝔩𝔢𝔞𝔰𝔢!</body></html>"
 //!    );
 //!
-//!    let nested_attachment = match nested_message.get_attachment(0).unwrap() {
-//!        MessagePart::Binary(v) => v,
-//!        _ => unreachable!(),
-//!    };
+//!    let nested_attachment = nested_message.get_attachment(0).unwrap().unwrap_binary();
 //!
 //!    assert_eq!(nested_attachment.len(), 42);
 //!
@@ -261,8 +263,9 @@
 pub mod decoders;
 pub mod parsers;
 
-use std::{borrow::Cow, collections::HashMap, fmt, slice::Iter};
+use std::{borrow::Cow, collections::HashMap, fmt};
 
+use decoders::html::{html_to_text, text_to_html};
 #[cfg(feature = "serde_support")]
 use serde::{Deserialize, Serialize};
 
@@ -270,94 +273,158 @@ use serde::{Deserialize, Serialize};
 #[derive(Debug, Default, PartialEq)]
 #[cfg_attr(feature = "serde_support", derive(Serialize, Deserialize))]
 pub struct Message<'x> {
-    #[cfg_attr(feature = "serde_support", serde(borrow))]
     #[cfg_attr(
         feature = "serde_support",
         serde(skip_serializing_if = "HashMap::is_empty")
     )]
     #[cfg_attr(feature = "serde_support", serde(default))]
-    headers: Headers<'x>,
+    pub headers_rfc: RfcHeaders<'x>,
+    #[cfg_attr(
+        feature = "serde_support",
+        serde(skip_serializing_if = "HashMap::is_empty")
+    )]
+    #[cfg_attr(feature = "serde_support", serde(default))]
+    pub headers_other: OtherHeaders<'x>,
+    #[cfg_attr(
+        feature = "serde_support",
+        serde(skip_serializing_if = "Vec::is_empty")
+    )]
+    #[cfg_attr(feature = "serde_support", serde(default))]
+    pub headers_offsets: Vec<HeaderOffset<'x>>,
 
     #[cfg_attr(
         feature = "serde_support",
         serde(skip_serializing_if = "Vec::is_empty")
     )]
     #[cfg_attr(feature = "serde_support", serde(default))]
-    html_body: Vec<InlinePart<'x>>,
+    pub html_body: Vec<MessagePartId>,
     #[cfg_attr(
         feature = "serde_support",
         serde(skip_serializing_if = "Vec::is_empty")
     )]
     #[cfg_attr(feature = "serde_support", serde(default))]
-    text_body: Vec<InlinePart<'x>>,
+    pub text_body: Vec<MessagePartId>,
     #[cfg_attr(
         feature = "serde_support",
         serde(skip_serializing_if = "Vec::is_empty")
     )]
     #[cfg_attr(feature = "serde_support", serde(default))]
-    attachments: Vec<MessagePart<'x>>,
-}
+    pub attachments: Vec<MessagePartId>,
 
-/// A text message part.
-#[derive(Debug, Default, PartialEq)]
-#[cfg_attr(feature = "serde_support", derive(Serialize, Deserialize))]
-pub struct TextPart<'x> {
     #[cfg_attr(
         feature = "serde_support",
-        serde(skip_serializing_if = "Option::is_none")
+        serde(skip_serializing_if = "Vec::is_empty")
     )]
     #[cfg_attr(feature = "serde_support", serde(default))]
-    headers: Option<Headers<'x>>,
-    contents: Cow<'x, str>,
-}
-
-/// A binary (`[u8]`) message part.
-#[derive(Debug, Default, PartialEq)]
-#[cfg_attr(feature = "serde_support", derive(Serialize, Deserialize))]
-pub struct BinaryPart<'x> {
-    #[cfg_attr(
-        feature = "serde_support",
-        serde(skip_serializing_if = "Option::is_none")
-    )]
-    #[cfg_attr(feature = "serde_support", serde(default))]
-    headers: Option<Headers<'x>>,
-    #[cfg_attr(feature = "serde_support", serde(with = "serde_bytes"))]
     #[cfg_attr(feature = "serde_support", serde(borrow))]
-    contents: Cow<'x, [u8]>,
+    pub parts: Vec<MessagePart<'x>>,
+
+    #[cfg_attr(feature = "serde_support", serde(default))]
+    pub structure: MessageStructure,
+
+    pub offset_header: usize,
+    pub offset_body: usize,
+    pub offset_end: usize,
+    #[cfg_attr(feature = "serde_support", serde(skip))]
+    pub raw_message: &'x [u8],
 }
 
-#[doc(hidden)]
+/// Body structure.
 #[derive(Debug, PartialEq)]
 #[cfg_attr(feature = "serde_support", derive(Serialize, Deserialize))]
-#[cfg_attr(feature = "serde_support", serde(untagged))]
-pub enum InlinePart<'x> {
-    Text(TextPart<'x>),
-    InlineBinary(u32),
+pub enum MessageStructure {
+    Part(MessagePartId),
+    List(Vec<MessageStructure>),
+    MultiPart((MessagePartId, Vec<MessageStructure>)),
 }
 
+impl Default for MessageStructure {
+    fn default() -> Self {
+        MessageStructure::Part(0)
+    }
+}
+
+// Part of the message.
+#[derive(Debug, Default, PartialEq)]
+#[cfg_attr(feature = "serde_support", derive(Serialize, Deserialize))]
+pub struct Part<'x, T> {
+    #[cfg_attr(
+        feature = "serde_support",
+        serde(skip_serializing_if = "HashMap::is_empty")
+    )]
+    #[cfg_attr(feature = "serde_support", serde(default))]
+    pub headers: RfcHeaders<'x>,
+
+    pub body: T,
+}
+
+impl<'x, T> Part<'x, T> {
+    pub fn new(headers: RfcHeaders<'x>, body: T) -> Self {
+        Self { headers, body }
+    }
+
+    pub fn get_body(&self) -> &T {
+        &self.body
+    }
+}
+
+/// Unique ID representing a MIME part within a message.
+pub type MessagePartId = usize;
+
 /// A text, binary or nested e-mail MIME message part.
 ///
 /// - Text: Any text/* part
-/// - Binary: Any other part type that is not text, usually attachments.
-/// - InlineBinary: Same as the Binary variant but an inline part according to RFC 8621, Section 4.1.4
-/// - Message: A nested RFC5322 message.
+/// - Binary: Any other part type that is not text.
+/// - Message: Nested RFC5322 message.
+/// - MultiPart: Multipart part.
 ///
 #[derive(Debug, PartialEq)]
 #[cfg_attr(feature = "serde_support", derive(Serialize, Deserialize))]
 pub enum MessagePart<'x> {
     /// Any text/* part
-    Text(TextPart<'x>),
+    Text(Part<'x, Cow<'x, str>>),
 
-    /// Any other part type that is not text, usually attachments.
+    /// A text/html part
+    Html(Part<'x, Cow<'x, str>>),
+
+    /// Any other part type that is not text.
     #[cfg_attr(feature = "serde_support", serde(borrow))]
-    Binary(BinaryPart<'x>),
+    Binary(Part<'x, Cow<'x, [u8]>>),
 
-    /// Same as the Binary variant but an inline part according to RFC 8621, Section 4.1.4
+    /// Any inline binary data that.
     #[cfg_attr(feature = "serde_support", serde(borrow))]
-    InlineBinary(BinaryPart<'x>),
+    InlineBinary(Part<'x, Cow<'x, [u8]>>),
 
-    /// A nested RFC5322 message.
-    Message(Message<'x>),
+    /// Nested RFC5322 message.
+    Message(Part<'x, Box<Message<'x>>>),
+
+    /// Multipart part
+    Multipart(RfcHeaders<'x>),
+}
+
+impl<'x> MessagePart<'x> {
+    pub fn unwrap_text(&self) -> &Part<'x, Cow<'x, str>> {
+        match self {
+            MessagePart::Text(part) => part,
+            MessagePart::Html(part) => part,
+            _ => panic!("Expected text part."),
+        }
+    }
+
+    pub fn unwrap_binary(&self) -> &Part<'x, Cow<'x, [u8]>> {
+        match self {
+            MessagePart::Binary(part) => part,
+            MessagePart::InlineBinary(part) => part,
+            _ => panic!("Expected binary part."),
+        }
+    }
+
+    pub fn unwrap_message(&self) -> &Part<'x, Box<Message<'x>>> {
+        match self {
+            MessagePart::Message(part) => part,
+            _ => panic!("Expected message part."),
+        }
+    }
 }
 
 /// An RFC5322 or RFC2369 internet address.
@@ -402,7 +469,24 @@ pub struct Group<'x> {
     pub addresses: Vec<Addr<'x>>,
 }
 
-pub type Headers<'x> = HashMap<HeaderName, HeaderValue<'x>>;
+pub type RfcHeaders<'x> = HashMap<HeaderName, HeaderValue<'x>>;
+pub type OtherHeaders<'x> = HashMap<Cow<'x, str>, HeaderValue<'x>>;
+
+/// Offset of a message element in the raw message.
+#[derive(Debug, PartialEq)]
+#[cfg_attr(feature = "serde_support", derive(Serialize, Deserialize))]
+pub struct HeaderOffset<'x> {
+    pub name: HeaderOffsetName<'x>,
+    pub start: usize,
+    pub end: usize,
+}
+
+#[derive(Debug, PartialEq)]
+#[cfg_attr(feature = "serde_support", derive(Serialize, Deserialize))]
+pub enum HeaderOffsetName<'x> {
+    Rfc(HeaderName),
+    Other(Cow<'x, str>),
+}
 
 /// A header field
 #[repr(u8)]
@@ -447,7 +531,7 @@ pub enum HeaderName {
     ListPost = 34,
     ListSubscribe = 35,
     ListUnsubscribe = 36,
-    Unsupported = 37,
+    Other = 37,
 }
 
 /// A parsed header value.
@@ -550,272 +634,309 @@ impl<'x> HeaderValue<'x> {
 #[derive(Debug, PartialEq)]
 #[cfg_attr(feature = "serde_support", derive(Serialize, Deserialize))]
 pub struct ContentType<'x> {
-    c_type: Cow<'x, str>,
+    pub c_type: Cow<'x, str>,
     #[cfg_attr(
         feature = "serde_support",
         serde(skip_serializing_if = "Option::is_none")
     )]
     #[cfg_attr(feature = "serde_support", serde(default))]
-    c_subtype: Option<Cow<'x, str>>,
+    pub c_subtype: Option<Cow<'x, str>>,
     #[cfg_attr(
         feature = "serde_support",
         serde(skip_serializing_if = "Option::is_none")
     )]
     #[cfg_attr(feature = "serde_support", serde(default))]
-    attributes: Option<HashMap<Cow<'x, str>, Cow<'x, str>>>,
+    pub attributes: Option<HashMap<Cow<'x, str>, Cow<'x, str>>>,
 }
 
 /// An RFC5322 datetime.
 #[derive(Debug, PartialEq)]
 #[cfg_attr(feature = "serde_support", derive(Serialize, Deserialize))]
 pub struct DateTime {
-    year: u32,
-    month: u32,
-    day: u32,
-    hour: u32,
-    minute: u32,
-    second: u32,
-    tz_before_gmt: bool,
-    tz_hour: u32,
-    tz_minute: u32,
+    pub year: u32,
+    pub month: u32,
+    pub day: u32,
+    pub hour: u32,
+    pub minute: u32,
+    pub second: u32,
+    pub tz_before_gmt: bool,
+    pub tz_hour: u32,
+    pub tz_minute: u32,
 }
 
 impl<'x> Message<'x> {
+    /// Returns an iterator over the RFC headers of this message.
+    pub fn get_headers_rfc(&self) -> impl Iterator<Item = (&HeaderName, &HeaderValue<'x>)> {
+        self.headers_rfc.iter()
+    }
+
     /// Returns the BCC header field
     pub fn get_bcc(&self) -> &HeaderValue<'x> {
-        self.headers
+        self.headers_rfc
             .get(&HeaderName::Bcc)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the CC header field
     pub fn get_cc(&self) -> &HeaderValue<'x> {
-        self.headers
+        self.headers_rfc
             .get(&HeaderName::Cc)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns all Comments header fields
     pub fn get_comments(&self) -> &HeaderValue<'x> {
-        self.headers
+        self.headers_rfc
             .get(&HeaderName::Comments)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the Date header field
     pub fn get_date(&self) -> Option<&DateTime> {
-        self.headers
+        self.headers_rfc
             .get(&HeaderName::Date)
             .and_then(|header| header.as_datetime_ref())
     }
 
     /// Returns the From header field
     pub fn get_from(&self) -> &HeaderValue<'x> {
-        self.headers
+        self.headers_rfc
             .get(&HeaderName::From)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns all In-Reply-To header fields
     pub fn get_in_reply_to(&self) -> &HeaderValue<'x> {
-        self.headers
+        self.headers_rfc
             .get(&HeaderName::InReplyTo)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns all Keywords header fields
     pub fn get_keywords(&self) -> &HeaderValue<'x> {
-        self.headers
+        self.headers_rfc
             .get(&HeaderName::Keywords)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the List-Archive header field
     pub fn get_list_archive(&self) -> &HeaderValue<'x> {
-        self.headers
+        self.headers_rfc
             .get(&HeaderName::ListArchive)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the List-Help header field
     pub fn get_list_help(&self) -> &HeaderValue<'x> {
-        self.headers
+        self.headers_rfc
             .get(&HeaderName::ListHelp)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the List-ID header field
     pub fn get_list_id(&self) -> &HeaderValue<'x> {
-        self.headers
+        self.headers_rfc
             .get(&HeaderName::ListId)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the List-Owner header field
     pub fn get_list_owner(&self) -> &HeaderValue<'x> {
-        self.headers
+        self.headers_rfc
             .get(&HeaderName::ListOwner)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the List-Post header field
     pub fn get_list_post(&self) -> &HeaderValue<'x> {
-        self.headers
+        self.headers_rfc
             .get(&HeaderName::ListPost)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the List-Subscribe header field
     pub fn get_list_subscribe(&self) -> &HeaderValue<'x> {
-        self.headers
+        self.headers_rfc
             .get(&HeaderName::ListSubscribe)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the List-Unsubscribe header field
     pub fn get_list_unsubscribe(&self) -> &HeaderValue<'x> {
-        self.headers
+        self.headers_rfc
             .get(&HeaderName::ListUnsubscribe)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the Message-ID header field
     pub fn get_message_id(&self) -> Option<&str> {
-        self.headers
+        self.headers_rfc
             .get(&HeaderName::MessageId)
             .and_then(|header| header.as_text_ref())
     }
 
     /// Returns the MIME-Version header field
     pub fn get_mime_version(&self) -> &HeaderValue<'x> {
-        self.headers
+        self.headers_rfc
             .get(&HeaderName::MimeVersion)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns all Received header fields
     pub fn get_received(&self) -> &HeaderValue<'x> {
-        self.headers
+        self.headers_rfc
             .get(&HeaderName::Received)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns all References header fields
     pub fn get_references(&self) -> &HeaderValue<'x> {
-        self.headers
+        self.headers_rfc
             .get(&HeaderName::References)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the Reply-To header field
     pub fn get_reply_to(&self) -> &HeaderValue<'x> {
-        self.headers
+        self.headers_rfc
             .get(&HeaderName::ReplyTo)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the Resent-BCC header field
     pub fn get_resent_bcc(&self) -> &HeaderValue<'x> {
-        self.headers
+        self.headers_rfc
             .get(&HeaderName::ResentBcc)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the Resent-CC header field
     pub fn get_resent_cc(&self) -> &HeaderValue<'x> {
-        self.headers
+        self.headers_rfc
             .get(&HeaderName::ResentTo)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns all Resent-Date header fields
     pub fn get_resent_date(&self) -> &HeaderValue<'x> {
-        self.headers
+        self.headers_rfc
             .get(&HeaderName::ResentDate)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the Resent-From header field
     pub fn get_resent_from(&self) -> &HeaderValue<'x> {
-        self.headers
+        self.headers_rfc
             .get(&HeaderName::ResentFrom)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns all Resent-Message-ID header fields
     pub fn get_resent_message_id(&self) -> &HeaderValue<'x> {
-        self.headers
+        self.headers_rfc
             .get(&HeaderName::ResentMessageId)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the Sender header field
     pub fn get_resent_sender(&self) -> &HeaderValue<'x> {
-        self.headers
+        self.headers_rfc
             .get(&HeaderName::ResentSender)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the Resent-To header field
     pub fn get_resent_to(&self) -> &HeaderValue<'x> {
-        self.headers
+        self.headers_rfc
             .get(&HeaderName::ResentTo)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns all Return-Path header fields
     pub fn get_return_path(&self) -> &HeaderValue<'x> {
-        self.headers
+        self.headers_rfc
             .get(&HeaderName::ReturnPath)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the Sender header field
     pub fn get_sender(&self) -> &HeaderValue<'x> {
-        self.headers
+        self.headers_rfc
             .get(&HeaderName::Sender)
             .unwrap_or(&HeaderValue::Empty)
     }
 
     /// Returns the Subject header field
     pub fn get_subject(&self) -> Option<&str> {
-        self.headers
+        self.headers_rfc
             .get(&HeaderName::Subject)
             .and_then(|header| header.as_text_ref())
     }
 
     /// Returns the To header field
     pub fn get_to(&self) -> &HeaderValue<'x> {
-        self.headers
+        self.headers_rfc
             .get(&HeaderName::To)
             .unwrap_or(&HeaderValue::Empty)
     }
 
-    fn get_body_part(&self, list: &'x [InlinePart<'x>], pos: usize) -> Option<&'x dyn BodyPart> {
-        match list.get(pos) {
-            Some(InlinePart::Text(v)) => Some(v),
-            Some(InlinePart::InlineBinary(v)) => match self.attachments.get(*v as usize)? {
-                MessagePart::Text(v) => Some(v),
-                MessagePart::Binary(v) | MessagePart::InlineBinary(v) => Some(v),
-                _ => None,
-            },
+    /// Returns a non-standard header field
+    pub fn get_other(&self, name: &'x str) -> &HeaderValue<'x> {
+        self.headers_other
+            .get(&Cow::from(name))
+            .unwrap_or(&HeaderValue::Empty)
+    }
+
+    fn get_part(&self, list: &'x [MessagePartId], pos: usize) -> Option<&'x dyn BodyPart> {
+        match self.parts.get(*list.get(pos)?)? {
+            MessagePart::Text(v) => Some(v),
+            MessagePart::Html(v) => Some(v),
+            MessagePart::Binary(v) => Some(v),
+            MessagePart::InlineBinary(v) => Some(v),
+            MessagePart::Message(v) => Some(v),
+            _ => None,
+        }
+    }
+
+    /// Returns the transformed contents an inline HTML body part by position
+    pub fn get_html_body(&'x self, pos: usize) -> Option<Cow<'x, str>> {
+        match self.parts.get(*self.html_body.get(pos)?)? {
+            MessagePart::Html(html) => Some(html.body.as_ref().into()),
+            MessagePart::Text(text) => Some(text_to_html(text.body.as_ref()).into()),
+            _ => None,
+        }
+    }
+
+    /// Returns the transformed contents an inline text body part by position
+    pub fn get_text_body(&'x self, pos: usize) -> Option<Cow<'x, str>> {
+        match self.parts.get(*self.text_body.get(pos)?)? {
+            MessagePart::Text(text) => Some(text.body.as_ref().into()),
+            MessagePart::Html(html) => Some(html_to_text(html.body.as_ref()).into()),
             _ => None,
         }
     }
 
     /// Returns an inline HTML body part by position
-    pub fn get_html_body(&self, pos: usize) -> Option<&dyn BodyPart> {
-        self.get_body_part(&self.html_body, pos)
+    pub fn get_html_part(&self, pos: usize) -> Option<&Part<Cow<'x, str>>> {
+        match self.parts.get(*self.html_body.get(pos)?)? {
+            MessagePart::Html(html) => Some(html),
+            MessagePart::Text(text) => Some(text),
+            _ => None,
+        }
     }
 
     /// Returns an inline text body part by position
-    pub fn get_text_body(&self, pos: usize) -> Option<&dyn BodyPart> {
-        self.get_body_part(&self.text_body, pos)
+    pub fn get_text_part(&self, pos: usize) -> Option<&Part<Cow<'x, str>>> {
+        match self.parts.get(*self.text_body.get(pos)?)? {
+            MessagePart::Html(html) => Some(html),
+            MessagePart::Text(text) => Some(text),
+            _ => None,
+        }
     }
 
     /// Returns an attacment by position
     pub fn get_attachment(&self, pos: usize) -> Option<&MessagePart<'x>> {
-        self.attachments.get(pos)
+        self.parts.get(*self.attachments.get(pos)?)
     }
 
     /// Returns the number of plain text body parts
@@ -844,8 +965,8 @@ impl<'x> Message<'x> {
     }
 
     /// Returns an Interator over the attachments
-    pub fn get_attachments(&self) -> Iter<MessagePart<'_>> {
-        self.attachments.iter()
+    pub fn get_attachments(&'x self) -> AttachmentIterator<'x> {
+        AttachmentIterator::new(self)
     }
 }
 
@@ -868,6 +989,157 @@ pub trait MimeHeaders<'x> {
 }
 
 impl<'x> MimeHeaders<'x> for Message<'x> {
+    fn get_content_description(&self) -> Option<&str> {
+        self.headers_rfc
+            .get(&HeaderName::ContentDescription)
+            .and_then(|header| header.as_text_ref())
+    }
+
+    fn get_content_disposition(&self) -> Option<&ContentType> {
+        self.headers_rfc
+            .get(&HeaderName::ContentDisposition)
+            .and_then(|header| header.as_content_type_ref())
+    }
+
+    fn get_content_id(&self) -> Option<&str> {
+        self.headers_rfc
+            .get(&HeaderName::ContentId)
+            .and_then(|header| header.as_text_ref())
+    }
+
+    fn get_content_transfer_encoding(&self) -> Option<&str> {
+        self.headers_rfc
+            .get(&HeaderName::ContentTransferEncoding)
+            .and_then(|header| header.as_text_ref())
+    }
+
+    fn get_content_type(&self) -> Option<&ContentType> {
+        self.headers_rfc
+            .get(&HeaderName::ContentType)
+            .and_then(|header| header.as_content_type_ref())
+    }
+
+    fn get_content_language(&self) -> &HeaderValue<'x> {
+        self.headers_rfc
+            .get(&HeaderName::ContentLanguage)
+            .unwrap_or(&HeaderValue::Empty)
+    }
+
+    fn get_content_location(&self) -> Option<&str> {
+        self.headers_rfc
+            .get(&HeaderName::ContentLocation)
+            .and_then(|header| header.as_text_ref())
+    }
+}
+
+/// An inline Text or Binary body part.
+pub trait BodyPart<'x>: fmt::Display + MimeHeaders<'x> {
+    /// Returns the body part's contents as a `u8` slice
+    fn get_contents(&'x self) -> &'x [u8];
+
+    /// Returns the body part's contents as a `str`
+    fn get_text_contents(&'x self) -> &'x str;
+
+    /// Returns the body part's length
+    fn len(&self) -> usize;
+
+    /// Returns `true` when the body part MIME type is text/*
+    fn is_text(&self) -> bool;
+
+    /// Returns `true` when the part is not text
+    fn is_binary(&self) -> bool;
+
+    /// Returns `true` when the body part is empty
+    fn is_empty(&self) -> bool {
+        self.len() == 0
+    }
+}
+
+impl<'x> BodyPart<'x> for Part<'x, Cow<'x, str>> {
+    fn get_contents(&'x self) -> &'x [u8] {
+        self.body.as_bytes()
+    }
+
+    fn is_text(&self) -> bool {
+        true
+    }
+
+    fn is_binary(&self) -> bool {
+        false
+    }
+
+    fn get_text_contents(&'x self) -> &'x str {
+        self.body.as_ref()
+    }
+
+    fn len(&self) -> usize {
+        self.body.len()
+    }
+}
+
+impl<'x> fmt::Display for Part<'x, Cow<'x, str>> {
+    fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
+        fmt.write_str(self.body.as_ref())
+    }
+}
+
+impl<'x> BodyPart<'x> for Part<'x, Cow<'x, [u8]>> {
+    fn get_contents(&'x self) -> &'x [u8] {
+        self.body.as_ref()
+    }
+
+    fn get_text_contents(&'x self) -> &'x str {
+        std::str::from_utf8(self.body.as_ref()).unwrap_or("")
+    }
+
+    fn is_text(&self) -> bool {
+        false
+    }
+
+    fn is_binary(&self) -> bool {
+        true
+    }
+
+    fn len(&self) -> usize {
+        self.body.len()
+    }
+}
+
+impl<'x> fmt::Display for Part<'x, Cow<'x, [u8]>> {
+    fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
+        fmt.write_str("[binary contents]")
+    }
+}
+
+impl<'x> BodyPart<'x> for Part<'x, Box<Message<'x>>> {
+    fn get_contents(&'x self) -> &'x [u8] {
+        self.body.raw_message
+    }
+
+    fn get_text_contents(&'x self) -> &'x str {
+        std::str::from_utf8(self.body.raw_message).unwrap_or("")
+    }
+
+    fn is_text(&self) -> bool {
+        false
+    }
+
+    fn is_binary(&self) -> bool {
+        true
+    }
+
+    fn len(&self) -> usize {
+        self.body.raw_message.len()
+    }
+}
+
+impl<'x> fmt::Display for Part<'x, Box<Message<'x>>> {
+    fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
+        fmt.write_str(String::from_utf8_lossy(self.body.raw_message).as_ref())
+    }
+}
+
+impl<'x, T> MimeHeaders<'x> for Part<'x, T> {
     fn get_content_description(&self) -> Option<&str> {
         self.headers
             .get(&HeaderName::ContentDescription)
@@ -911,196 +1183,21 @@ impl<'x> MimeHeaders<'x> for Message<'x> {
     }
 }
 
-/// An inline Text or Binary body part.
-pub trait BodyPart<'x>: fmt::Display {
-    /// Returns the body part's contents as a `u8` slice
-    fn get_contents(&'x self) -> &'x [u8];
-
-    /// Returns the body part's contents as a `str`
-    fn get_text_contents(&'x self) -> &'x str;
-
-    /// Returns the body part's length
-    fn len(&self) -> usize;
-
-    /// Returns `true` when the body part MIME type is text/*
-    fn is_text(&self) -> bool;
-
-    /// Returns `true` when the part is not text
-    fn is_binary(&self) -> bool;
-
-    /// Returns `true` when the body part is empty
-    fn is_empty(&self) -> bool {
-        self.len() == 0
-    }
-}
-
-impl<'x> BodyPart<'x> for TextPart<'x> {
-    fn get_contents(&'x self) -> &'x [u8] {
-        self.contents.as_bytes()
-    }
-
-    fn is_text(&self) -> bool {
-        true
-    }
-
-    fn is_binary(&self) -> bool {
-        false
-    }
-
-    fn get_text_contents(&'x self) -> &'x str {
-        self.contents.as_ref()
-    }
-
-    fn len(&self) -> usize {
-        self.contents.len()
-    }
-}
-
-impl<'x> fmt::Display for TextPart<'x> {
-    fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
-        fmt.write_str(self.contents.as_ref())
-    }
-}
-
-impl<'x> BodyPart<'x> for BinaryPart<'x> {
-    fn get_contents(&'x self) -> &'x [u8] {
-        self.contents.as_ref()
-    }
-
-    fn get_text_contents(&'x self) -> &'x str {
-        std::str::from_utf8(self.contents.as_ref()).unwrap_or("")
-    }
-
-    fn is_text(&self) -> bool {
-        false
-    }
-
-    fn is_binary(&self) -> bool {
-        true
-    }
-
-    fn len(&self) -> usize {
-        self.contents.len()
-    }
-}
-
-impl<'x> fmt::Display for BinaryPart<'x> {
-    fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
-        fmt.write_str("[binary contents]")
-    }
-}
-
-impl<'x> MimeHeaders<'x> for TextPart<'x> {
-    fn get_content_description(&self) -> Option<&str> {
-        self.headers.as_ref().and_then(|h| {
-            h.get(&HeaderName::ContentDescription)
-                .and_then(|header| header.as_text_ref())
-        })
-    }
-
-    fn get_content_disposition(&self) -> Option<&ContentType> {
-        self.headers.as_ref().and_then(|h| {
-            h.get(&HeaderName::ContentDisposition)
-                .and_then(|header| header.as_content_type_ref())
-        })
-    }
-
-    fn get_content_id(&self) -> Option<&str> {
-        self.headers.as_ref().and_then(|h| {
-            h.get(&HeaderName::ContentId)
-                .and_then(|header| header.as_text_ref())
-        })
-    }
-
-    fn get_content_transfer_encoding(&self) -> Option<&str> {
-        self.headers.as_ref().and_then(|h| {
-            h.get(&HeaderName::ContentTransferEncoding)
-                .and_then(|header| header.as_text_ref())
-        })
-    }
-
-    fn get_content_type(&self) -> Option<&ContentType> {
-        self.headers.as_ref().and_then(|h| {
-            h.get(&HeaderName::ContentType)
-                .and_then(|header| header.as_content_type_ref())
-        })
-    }
-
-    fn get_content_language(&self) -> &HeaderValue<'x> {
-        self.headers.as_ref().map_or(&HeaderValue::Empty, |h| {
-            h.get(&HeaderName::ContentLanguage)
-                .unwrap_or(&HeaderValue::Empty)
-        })
-    }
-
-    fn get_content_location(&self) -> Option<&str> {
-        self.headers.as_ref().and_then(|h| {
-            h.get(&HeaderName::ContentLocation)
-                .and_then(|header| header.as_text_ref())
-        })
-    }
-}
-
-impl<'x> MimeHeaders<'x> for BinaryPart<'x> {
-    fn get_content_description(&self) -> Option<&str> {
-        self.headers.as_ref().and_then(|h| {
-            h.get(&HeaderName::ContentDescription)
-                .and_then(|header| header.as_text_ref())
-        })
-    }
-
-    fn get_content_disposition(&self) -> Option<&ContentType> {
-        self.headers.as_ref().and_then(|h| {
-            h.get(&HeaderName::ContentDisposition)
-                .and_then(|header| header.as_content_type_ref())
-        })
-    }
-
-    fn get_content_id(&self) -> Option<&str> {
-        self.headers.as_ref().and_then(|h| {
-            h.get(&HeaderName::ContentId)
-                .and_then(|header| header.as_text_ref())
-        })
-    }
-
-    fn get_content_transfer_encoding(&self) -> Option<&str> {
-        self.headers.as_ref().and_then(|h| {
-            h.get(&HeaderName::ContentTransferEncoding)
-                .and_then(|header| header.as_text_ref())
-        })
-    }
-
-    fn get_content_type(&self) -> Option<&ContentType> {
-        self.headers.as_ref().and_then(|h| {
-            h.get(&HeaderName::ContentType)
-                .and_then(|header| header.as_content_type_ref())
-        })
-    }
-
-    fn get_content_language(&self) -> &HeaderValue<'x> {
-        self.headers.as_ref().map_or(&HeaderValue::Empty, |h| {
-            h.get(&HeaderName::ContentLanguage)
-                .unwrap_or(&HeaderValue::Empty)
-        })
-    }
-
-    fn get_content_location(&self) -> Option<&str> {
-        self.headers.as_ref().and_then(|h| {
-            h.get(&HeaderName::ContentLocation)
-                .and_then(|header| header.as_text_ref())
-        })
-    }
-}
-
 #[doc(hidden)]
 pub struct BodyPartIterator<'x> {
     message: &'x Message<'x>,
-    list: &'x [InlinePart<'x>],
+    list: &'x [MessagePartId],
+    pos: isize,
+}
+
+#[doc(hidden)]
+pub struct AttachmentIterator<'x> {
+    message: &'x Message<'x>,
     pos: isize,
 }
 
 impl<'x> BodyPartIterator<'x> {
-    fn new(message: &'x Message<'x>, list: &'x [InlinePart<'x>]) -> BodyPartIterator<'x> {
+    fn new(message: &'x Message<'x>, list: &'x [MessagePartId]) -> BodyPartIterator<'x> {
         BodyPartIterator {
             message,
             list,
@@ -1114,7 +1211,22 @@ impl<'x> Iterator for BodyPartIterator<'x> {
 
     fn next(&mut self) -> Option<Self::Item> {
         self.pos += 1;
-        self.message.get_body_part(self.list, self.pos as usize)
+        self.message.get_part(self.list, self.pos as usize)
+    }
+}
+
+impl<'x> AttachmentIterator<'x> {
+    fn new(message: &'x Message<'x>) -> AttachmentIterator<'x> {
+        AttachmentIterator { message, pos: -1 }
+    }
+}
+
+impl<'x> Iterator for AttachmentIterator<'x> {
+    type Item = &'x MessagePart<'x>;
+
+    fn next(&mut self) -> Option<Self::Item> {
+        self.pos += 1;
+        self.message.get_attachment(self.pos as usize)
     }
 }
 
