@@ -13,19 +13,16 @@ use std::borrow::Cow;
 
 use crate::{
     decoders::{
-        base64::decode_base64,
-        charsets::map::get_charset_decoder,
-        html::{html_to_text, text_to_html},
-        quoted_printable::decode_quoted_printable,
-        DecodeFnc, DecodeResult,
+        base64::decode_base64, charsets::map::get_charset_decoder,
+        quoted_printable::decode_quoted_printable, DecodeFnc, DecodeResult,
     },
-    BinaryPart, ContentType, HeaderName, HeaderValue, Headers, InlinePart, Message, MessagePart,
-    TextPart,
+    ContentType, HeaderName, HeaderValue, Message, MessagePart, MessagePartId, MessageStructure,
+    Part, RfcHeaders,
 };
 
 use super::{
     header::parse_headers,
-    mime::{get_bytes_to_boundary, seek_next_part, skip_crlf, skip_multipart_end},
+    mime::{get_bytes_to_boundary, seek_crlf_end, seek_next_part, skip_crlf, skip_multipart_end},
 };
 
 #[derive(Debug, PartialEq)]
@@ -42,6 +39,12 @@ enum MimeType {
     Other,
 }
 
+impl Default for MimeType {
+    fn default() -> Self {
+        MimeType::Message
+    }
+}
+
 fn result_to_string<'x>(
     result: DecodeResult,
     data: &'x [u8],
@@ -111,6 +114,7 @@ fn get_mime_type(
     }
 }
 
+#[derive(Default)]
 struct MessageParserState {
     mime_type: MimeType,
     mime_boundary: Option<Vec<u8>>,
@@ -120,6 +124,8 @@ struct MessageParserState {
     text_parts: usize,
     need_html_body: bool,
     need_text_body: bool,
+    header_id: Option<MessagePartId>,
+    structure: Vec<MessageStructure>,
 }
 
 impl MessageParserState {
@@ -133,6 +139,19 @@ impl MessageParserState {
             text_parts: 0,
             need_text_body: true,
             need_html_body: true,
+            ..Default::default()
+        }
+    }
+
+    pub fn get_structure(&mut self) -> MessageStructure {
+        if let Some(header_id) = self.header_id {
+            MessageStructure::MultiPart((header_id, std::mem::take(&mut self.structure)))
+        } else if self.structure.len() > 1 {
+            MessageStructure::List(std::mem::take(&mut self.structure))
+        } else if self.structure.len() == 1 {
+            self.structure.pop().unwrap()
+        } else {
+            MessageStructure::List(vec![])
         }
     }
 }
@@ -151,14 +170,13 @@ impl<'x> MessageStream<'x> {
 impl<'x> Message<'x> {
     fn new() -> Message<'x> {
         Message {
-            headers: Headers::new(),
             ..Default::default()
         }
     }
 
     /// Returns `false` if at least one header field was successfully parsed.
     pub fn is_empty(&self) -> bool {
-        self.headers.is_empty()
+        self.headers_rfc.is_empty() && self.headers_other.is_empty()
     }
 
     /// Parses a byte slice containing the RFC5322 raw message and returns a
@@ -167,29 +185,40 @@ impl<'x> Message<'x> {
     /// This function never panics, a best-effort is made to parse the message and
     /// if no headers are found None is returned.
     ///
-    pub fn parse(bytes: &'x [u8]) -> Option<Message<'x>> {
-        let mut stream = MessageStream::new(bytes);
+    pub fn parse(raw_message: &'x [u8]) -> Option<Message<'x>> {
+        let mut stream = MessageStream::new(raw_message);
 
         let mut message = Message::new();
         let mut message_stack = Vec::new();
+        let mut message_header_stack = Vec::new();
 
         let mut state = MessageParserState::new();
         let mut state_stack = Vec::new();
 
-        let mut mime_part_header = Headers::new();
+        let mut mime_part_header = RfcHeaders::new();
+
+        message.raw_message = raw_message;
 
         'outer: loop {
-            // Obtain reference to either the message or the MIME part's header
-            let header = if let MimeType::Message = state.mime_type {
-                &mut message.headers
-            } else {
-                &mut mime_part_header
-            };
-
             // Parse headers
-            if !parse_headers(header, &mut stream) {
-                break;
-            }
+            let (is_message, header) = if let MimeType::Message = state.mime_type {
+                message.offset_header = stream.pos;
+                if !parse_headers(
+                    &mut message.headers_rfc,
+                    Some(&mut message.headers_other),
+                    Some(&mut message.headers_offsets),
+                    &mut stream,
+                ) {
+                    break;
+                }
+                message.offset_body = seek_crlf_end(&stream, stream.pos);
+                (true, &mut message.headers_rfc)
+            } else {
+                if !parse_headers(&mut mime_part_header, None, None, &mut stream) {
+                    break;
+                }
+                (false, &mut mime_part_header)
+            };
 
             state.parts += 1;
 
@@ -212,13 +241,32 @@ impl<'x> Message<'x> {
                                 || mime_type == MimeType::MultipartAlernative,
                             mime_type,
                             mime_boundary: mime_boundary.into(),
-                            parts: 0,
                             html_parts: message.html_body.len(),
                             text_parts: message.text_body.len(),
                             need_html_body: state.need_html_body,
                             need_text_body: state.need_text_body,
+                            header_id: if !is_message {
+                                Some(message.parts.len())
+                            } else {
+                                None
+                            },
+                            ..Default::default()
                         };
-                        mime_part_header.clear();
+                        if !is_message {
+                            if mime_part_header.is_empty() {
+                                mime_part_header.insert(
+                                    HeaderName::ContentType,
+                                    HeaderValue::ContentType(ContentType {
+                                        c_type: "text".into(),
+                                        c_subtype: Some("plain".into()),
+                                        attributes: None,
+                                    }),
+                                );
+                            }
+                            message.parts.push(MessagePart::Multipart(std::mem::take(
+                                &mut mime_part_header,
+                            )));
+                        }
                         state_stack.push(state);
                         state = new_state;
                         skip_crlf(&mut stream);
@@ -232,14 +280,21 @@ impl<'x> Message<'x> {
                 let new_state = MessageParserState {
                     mime_type: MimeType::Message,
                     mime_boundary: state.mime_boundary.take(),
-                    in_alternative: false,
-                    parts: 0,
-                    html_parts: 0,
-                    text_parts: 0,
                     need_html_body: true,
                     need_text_body: true,
+                    ..Default::default()
                 };
-                mime_part_header.clear();
+                if mime_part_header.is_empty() {
+                    mime_part_header.insert(
+                        HeaderName::ContentType,
+                        HeaderValue::ContentType(ContentType {
+                            c_type: "message".into(),
+                            c_subtype: Some("rfc822".into()),
+                            attributes: None,
+                        }),
+                    );
+                }
+                message_header_stack.push(std::mem::take(&mut mime_part_header));
                 message_stack.push(message);
                 state_stack.push(state);
                 message = Message::new();
@@ -354,58 +409,53 @@ impl<'x> Message<'x> {
             };
 
             if is_text {
-                let text_part = TextPart {
-                    contents: result_to_string(bytes, stream.data, content_type),
-                    headers: if !mime_part_header.is_empty() {
-                        Some(std::mem::take(&mut mime_part_header))
-                    } else {
-                        None
-                    },
+                let is_html = mime_type == MimeType::TextHtml;
+                let text_part = Part {
+                    body: result_to_string(bytes, stream.data, content_type),
+                    headers: std::mem::take(&mut mime_part_header),
                 };
 
-                let is_html = mime_type == MimeType::TextHtml;
-
                 if add_to_html && !is_html {
-                    message.html_body.push(InlinePart::Text(TextPart {
-                        headers: None,
-                        contents: text_to_html(&text_part.contents).into(),
-                    }));
+                    message.html_body.push(message.parts.len());
                 } else if add_to_text && is_html {
-                    message.text_body.push(InlinePart::Text(TextPart {
-                        headers: None,
-                        contents: html_to_text(&text_part.contents).into(),
-                    }));
+                    message.text_body.push(message.parts.len());
                 }
 
                 if add_to_html && is_html {
-                    message.html_body.push(InlinePart::Text(text_part));
+                    message.html_body.push(message.parts.len());
                 } else if add_to_text && !is_html {
-                    message.text_body.push(InlinePart::Text(text_part));
+                    message.text_body.push(message.parts.len());
                 } else {
-                    message.attachments.push(MessagePart::Text(text_part));
+                    message.attachments.push(message.parts.len());
                 }
+                state
+                    .structure
+                    .push(MessageStructure::Part(message.parts.len()));
+                message.parts.push(if is_html {
+                    MessagePart::Html(text_part)
+                } else {
+                    MessagePart::Text(text_part)
+                });
             } else {
-                let binary_part = BinaryPart {
-                    headers: if !mime_part_header.is_empty() {
-                        Some(std::mem::take(&mut mime_part_header))
-                    } else {
-                        None
-                    },
-                    contents: result_to_bytes(bytes, stream.data),
-                };
-
                 if add_to_html {
-                    message
-                        .html_body
-                        .push(InlinePart::InlineBinary(message.attachments.len() as u32));
+                    message.html_body.push(message.parts.len());
                 }
                 if add_to_text {
-                    message
-                        .text_body
-                        .push(InlinePart::InlineBinary(message.attachments.len() as u32));
+                    message.text_body.push(message.parts.len());
                 }
 
-                message.attachments.push(if !is_inline {
+                message.attachments.push(message.parts.len());
+
+                state
+                    .structure
+                    .push(MessageStructure::Part(message.parts.len()));
+
+                let binary_part = Part::new(
+                    std::mem::take(&mut mime_part_header),
+                    result_to_bytes(bytes, stream.data),
+                );
+
+                message.parts.push(if !is_inline {
                     MessagePart::Binary(binary_part)
                 } else {
                     MessagePart::InlineBinary(binary_part)
@@ -414,6 +464,7 @@ impl<'x> Message<'x> {
 
             if state.mime_boundary.is_some() {
                 // Currently processing a MIME part
+                let mut last_part_offset = stream.pos;
 
                 'inner: loop {
                     if let MimeType::Message = state.mime_type {
@@ -421,7 +472,15 @@ impl<'x> Message<'x> {
                         if let (Some(mut prev_message), Some(mut prev_state)) =
                             (message_stack.pop(), state_stack.pop())
                         {
-                            prev_message.attachments.push(MessagePart::Message(message));
+                            message.structure = state.get_structure();
+                            message.offset_end = seek_crlf_end(&stream, last_part_offset);
+                            message.raw_message =
+                                &raw_message[message.offset_header..message.offset_end];
+                            prev_message.attachments.push(prev_message.parts.len());
+                            prev_message.parts.push(MessagePart::Message(Part::new(
+                                message_header_stack.pop().unwrap(),
+                                Box::new(message),
+                            )));
                             message = prev_message;
                             prev_state.mime_boundary = state.mime_boundary;
                             state = prev_state;
@@ -442,16 +501,8 @@ impl<'x> Message<'x> {
                             if state.text_parts == message.text_body.len()
                                 && state.html_parts != message.html_body.len()
                             {
-                                for part in message.html_body[state.html_parts..].iter() {
-                                    message.text_body.push(match part {
-                                        InlinePart::Text(part) => InlinePart::Text(TextPart {
-                                            headers: None,
-                                            contents: html_to_text(&part.contents).into(),
-                                        }),
-                                        InlinePart::InlineBinary(index) => {
-                                            InlinePart::InlineBinary(*index)
-                                        }
-                                    });
+                                for &part_id in &message.html_body[state.html_parts..] {
+                                    message.text_body.push(part_id);
                                 }
                             }
 
@@ -459,23 +510,19 @@ impl<'x> Message<'x> {
                             if state.html_parts == message.html_body.len()
                                 && state.text_parts != message.text_body.len()
                             {
-                                for part in message.text_body[state.text_parts..].iter() {
-                                    message.html_body.push(match part {
-                                        InlinePart::Text(part) => InlinePart::Text(TextPart {
-                                            headers: None,
-                                            contents: text_to_html(&part.contents).into(),
-                                        }),
-                                        InlinePart::InlineBinary(index) => {
-                                            InlinePart::InlineBinary(*index)
-                                        }
-                                    });
+                                for &part_id in &message.text_body[state.html_parts..] {
+                                    message.html_body.push(part_id);
                                 }
                             }
                         }
 
-                        if let Some(prev_state) = state_stack.pop() {
+                        if let Some(mut prev_state) = state_stack.pop() {
+                            // Add headers and substructure to parent part
+                            prev_state.structure.push(state.get_structure());
+
                             // Restore ancestor's state
                             state = prev_state;
+                            last_part_offset = stream.pos;
 
                             if let Some(ref mime_boundary) = state.mime_boundary {
                                 // Ancestor has a MIME boundary, seek it.
@@ -496,13 +543,25 @@ impl<'x> Message<'x> {
             }
         }
 
-        while let Some(mut prev_message) = message_stack.pop() {
+        while let (Some(mut prev_message), Some(prev_state)) =
+            (message_stack.pop(), state_stack.pop())
+        {
+            message.offset_end = stream.pos;
             if !message.is_empty() {
-                prev_message.attachments.push(MessagePart::Message(message));
+                message.structure = state.get_structure();
+                prev_message.attachments.push(prev_message.parts.len());
+                prev_message.parts.push(MessagePart::Message(Part::new(
+                    message_header_stack.pop().unwrap(),
+                    Box::new(message),
+                )));
             }
             message = prev_message;
+            state = prev_state;
         }
 
+        message.structure = state.get_structure();
+        message.offset_end = stream.pos;
+
         if !message.is_empty() {
             Some(message)
         } else {
@@ -514,12 +573,12 @@ impl<'x> Message<'x> {
 mod tests {
     use std::{fs, path::PathBuf};
 
-    use crate::parsers::message::Message;
+    use crate::{parsers::message::Message, MessagePart};
+
+    const SEPARATOR: &[u8] = "\n---- EXPECTED STRUCTURE ----\n".as_bytes();
 
     #[test]
     fn parse_full_messages() {
-        const SEPARATOR: &[u8] = "\n---- EXPECTED STRUCTURE ----\n".as_bytes();
-
         for test_suite in ["rfc", "legacy", "thirdparty", "malformed"] {
             let mut test_dir = PathBuf::from(env!("CARGO_MANIFEST_DIR"));
             test_dir.push("tests");
@@ -549,7 +608,20 @@ mod tests {
                     tests_run += 1;
 
                     let input = input.split_at_mut(pos);
-                    let message = Message::parse(input.0).unwrap();
+                    let mut message = Message::parse(input.0).unwrap();
+
+                    // Remove raw message references before comparing
+                    message.raw_message = &[];
+                    for parts in &mut message.parts {
+                        if let MessagePart::Message(part) = parts {
+                            part.body.raw_message = &[];
+                            for parts in &mut part.body.parts {
+                                if let MessagePart::Message(part) = parts {
+                                    part.body.raw_message = &[];
+                                }
+                            }
+                        }
+                    }
 
                     assert_eq!(
                         message,
@@ -569,12 +641,43 @@ mod tests {
         }
     }
 
-    /*
+    /*#[test]
+    fn message_to_yaml() {
+        let mut file_name = PathBuf::from(env!("CARGO_MANIFEST_DIR"));
+        file_name.push("tests");
+        file_name.push("rfc");
+        file_name.push("007.txt");
+
+        let mut input = fs::read(&file_name).unwrap();
+        let mut pos = 0;
+
+        for sep_pos in 0..input.len() {
+            if input[sep_pos..sep_pos + SEPARATOR.len()].eq(SEPARATOR) {
+                pos = sep_pos;
+                break;
+            }
+        }
+
+        assert!(
+            pos > 0,
+            "Failed to find separator in test file '{}'.",
+            file_name.display()
+        );
+
+        let input = input.split_at_mut(pos).0;
+        let message = Message::parse(input).unwrap();
+        let result = serde_yaml::to_string(&message).unwrap();
+
+        fs::write("test.yaml", &result).unwrap();
+
+        println!("{:}", result);
+    }
+
     #[test]
     fn generate_test_samples() {
         const SEPARATOR: &[u8] = "\n---- EXPECTED STRUCTURE ----\n".as_bytes();
 
-        for test_suite in ["malformed", "legacy", "rfc", "thirdparty"] {
+        for test_suite in ["legacy", "malformed", "rfc", "thirdparty"] {
             let mut test_dir = PathBuf::from(env!("CARGO_MANIFEST_DIR"));
             test_dir.push("tests");
             test_dir.push(test_suite);
@@ -611,8 +714,8 @@ mod tests {
                 output.extend_from_slice(input.0);
                 output.extend_from_slice(SEPARATOR);
                 output.extend_from_slice(
-                    serde_json::to_string_pretty(&Message::parse(input.0))
-                        .unwrap_or_else(|_| "".to_string())
+                    serde_json::to_string_pretty(&Message::parse(input.0).unwrap())
+                        .unwrap()
                         .as_bytes(),
                 );
                 fs::write(file_name.as_ref().unwrap().path(), &output).unwrap();
